// Fitting MGLMM Unit gamma 
#include <TMB.hpp>
template<class Type>
Type objective_function<Type>::operator() ()
{
  using namespace density;
  DATA_VECTOR(Y1);
  DATA_VECTOR(Y2);
  DATA_VECTOR(Y3);
  DATA_VECTOR(Y4);
  DATA_VECTOR(Y5);
  DATA_MATRIX(X);
  PARAMETER_VECTOR(beta1);
  PARAMETER_VECTOR(beta2);
  PARAMETER_VECTOR(beta3);
  PARAMETER_VECTOR(beta4);
  PARAMETER_VECTOR(beta5);
  PARAMETER_MATRIX(U);
  PARAMETER_VECTOR(rho);
  PARAMETER_VECTOR(sigma);
  PARAMETER_VECTOR(phi);
  
  // Preparing
  vector<Type> rho_temp(10);
  rho_temp = rho;
  
  vector<Type> sigma_temp(5);
  sigma_temp = sigma;
  
  vector<Type> mu1(Y1.size());
  vector<Type> mu2(Y2.size());
  vector<Type> mu3(Y3.size());
  vector<Type> mu4(Y4.size());
  vector<Type> mu5(Y5.size());
  
  vector<Type> tau1(Y1.size());
  vector<Type> tau2(Y2.size());
  vector<Type> tau3(Y3.size());
  vector<Type> tau4(Y4.size());
  vector<Type> tau5(Y5.size());
  
  vector<Type> phi_temp(5);
  phi_temp = phi;
  
  // Linear predictor for mean model
  mu1 = exp(X*beta1 + U.col(0).array())/(1 + exp(X*beta1 + U.col(0).array()));
  mu2 = exp(X*beta2 + U.col(1).array())/(1 + exp(X*beta2 + U.col(1).array())); 
  mu3 = exp(X*beta3 + U.col(2).array())/(1 + exp(X*beta3 + U.col(2).array())); 
  mu4 = exp(X*beta4 + U.col(3).array())/(1 + exp(X*beta4 + U.col(3).array())); 
  mu5 = exp(X*beta5 + U.col(4).array())/(1 + exp(X*beta5 + U.col(4).array())); 
  
  // Unit gamma parametrization
  tau1 = pow(mu1, 1/exp(phi_temp(0)))/(1 - pow(mu1, 1/exp(phi_temp(0))));
  tau2 = pow(mu2, 1/exp(phi_temp(1)))/(1 - pow(mu2, 1/exp(phi_temp(1))));
  tau3 = pow(mu3, 1/exp(phi_temp(2)))/(1 - pow(mu3, 1/exp(phi_temp(2))));
  tau4 = pow(mu4, 1/exp(phi_temp(3)))/(1 - pow(mu4, 1/exp(phi_temp(3))));
  tau5 = pow(mu5, 1/exp(phi_temp(4)))/(1 - pow(mu5, 1/exp(phi_temp(4))));
  
  // Full log-likelihood   
  Type nll1 = 0;
  for(int i=0; i<Y1.size(); i++)
    nll1 -= exp(phi_temp(0))*log(tau1(i))-1*lgamma(exp(phi_temp(0)))+(tau1(i)-1)*log(Y1(i))+(exp(phi_temp(0))-1)*log(-log(Y1(i)));
  
  Type nll2 = 0;
  for(int i=0; i<Y2.size(); i++)
    nll2 -= exp(phi_temp(1))*log(tau2(i))-1*lgamma(exp(phi_temp(1)))+(tau2(i)-1)*log(Y2(i))+(exp(phi_temp(1))-1)*log(-log(Y2(i)));
  
  Type nll3 = 0;
  for(int i=0; i<Y3.size(); i++)
    nll3 -= exp(phi_temp(2))*log(tau3(i))-1*lgamma(exp(phi_temp(2)))+(tau3(i)-1)*log(Y3(i))+(exp(phi_temp(2))-1)*log(-log(Y3(i)));
  
  Type nll4 = 0;
  for(int i=0; i<Y4.size(); i++)
    nll4 -= exp(phi_temp(3))*log(tau4(i))-1*lgamma(exp(phi_temp(3)))+(tau4(i)-1)*log(Y4(i))+(exp(phi_temp(3))-1)*log(-log(Y4(i)));
  
  Type nll5 = 0;
  for(int i=0; i<Y5.size(); i++)
    nll5 -= exp(phi_temp(4))*log(tau5(i))-1*lgamma(exp(phi_temp(4)))+(tau5(i)-1)*log(Y5(i))+(exp(phi_temp(4))-1)*log(-log(Y5(i)));
  
  Type nll = 0;
  for(int i = 0; i < Y1.size(); i++)
    nll += VECSCALE(UNSTRUCTURED_CORR(rho_temp), sigma_temp)(U.row(i));
  
  matrix<Type> Cor(5,5);
  Cor = UNSTRUCTURED_CORR(rho_temp).cov();
  REPORT(Cor);
  REPORT(sigma_temp);
  REPORT(exp(phi));
  return nll1 + nll2 + nll3 + nll4 + nll5 + nll;
}
